"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const react_1 = __importDefault(require("react"));
const objss_1 = __importDefault(require("objss"));
const css_1 = __importDefault(require("styled-jsx/css"));
const classnames_1 = __importDefault(require("classnames"));
const react_2 = require("react");
// Theme support
const Theme = react_2.createContext({});
const useTheme = () => react_2.useContext(Theme);
exports.ThemeProvider = ({ children, value, theme }) => {
    if (!value && !theme) {
        throw new Error('Must provide either value or theme.');
    }
    // Support both <ThemeProvider value={...}> and <ThemeProvider theme={...}> (latter being styled-components standard)
    return react_1.default.createElement(Theme.Provider, { value: value || theme }, children);
};
const createMobileQuery = rules => css_1.default.resolve `
  @media (min-width: 40em) {
    ${objss_1.default(rules)}
  }
`;
const createTabletQuery = rules => css_1.default.resolve `
  @media (min-width: 52em) {
    ${objss_1.default(rules)}
  }
`;
const styler = (props, opts, theme) => {
    const styleObjects = [];
    const mediaQueries = [];
    opts.forEach(opt => {
        const x = opt({ ...props, theme });
        Object.entries(x).forEach(([key, value]) => {
            if (key.startsWith('@media')) {
                // Brittle check to generate the correct media query
                if (key.includes('40em')) {
                    mediaQueries.push(createMobileQuery(value));
                }
                else {
                    mediaQueries.push(createTabletQuery(value));
                }
            }
            else {
                // Not a media query, just regular styling
                styleObjects.push({ [key]: value });
            }
        });
    });
    const styles = Object.assign({}, ...styleObjects);
    return [
        css_1.default.resolve `
      ${objss_1.default(styles)}
    `,
        ...mediaQueries
    ];
};
const HOC = (Component, opts) => {
    const Comp = ({ className, children, ...props }) => {
        const theme = useTheme();
        const styles = styler(props, opts, theme);
        return (react_1.default.createElement(Component, Object.assign({}, props, { className: classnames_1.default(className, [...styles.map(s => s.className)]) }),
            children,
            styles.map((s, i) => {
                return (react_1.default.createElement(react_1.default.Fragment, { key: `styled-system-${i}` }, s.styles));
            })));
    };
    return Comp;
};
exports.default = HOC;
